@@ -1,11 +1,12 @@
-from synoptic.datamodel import \
-        Item, ItemVersion, Tag, ViewOrdering, ViewOrderingEntry, \
-        store_itemversion,  \
-        get_current_itemversions_join, \
-        query_itemversions
-
+from synoptic.datamodel import (
+        ItemVersion, Tag, ViewOrdering,
+        store_itemversion,
+        get_current_itemversions_join,
+        query_itemversions)
+import json
 
 
+# {{{ static file retrieval
 
 def get_static_file(filename):
     from os.path import splitext, join, normpath
@@ -37,7 +38,7 @@ def get_static_file(filename):
 
     return (data, mimetypes.get(ext, "application/octet-stream"))
 
-
+# }}}
 
 
 # {{{ import backup/initial content
@@ -65,7 +66,7 @@ def import_file(dbsession, text):
 
         idx += 1  # skip separator
 
-        store_itemversion(dbsession, 
+        store_itemversion(dbsession,
                 contents="\n".join(body), tags=tags, timestamp=timestamp)
 
     dbsession.commit()
@@ -73,16 +74,15 @@ def import_file(dbsession, text):
 # }}}
 
 
-
+# {{{ random usefulness
 
 def kill_hms_in_time_struct(t_struct):
     t_struct = list(t_struct)
-    t_struct[3:6] = (0,0,0)
+    t_struct[3:6] = (0, 0, 0)
     t_struct[8] = -1
     return t_struct
 
 
-
 def get_google_calendar_tag(entry_dict):
     from hashlib import md5
     md5_obj = md5()
@@ -90,6 +90,59 @@ def get_google_calendar_tag(entry_dict):
     return "gcal-"+md5_obj.hexdigest()[:10]
 
 
+def parse_datetime(s, rel_to=None, use_utc=False):
+    import parsedatetime as pdt
+
+    us_ptc = pdt.Constants("en_US")
+    us_cal = pdt.Calendar(us_ptc)
+
+    native_cal = pdt.Calendar()
+    import datetime
+    import time
+
+    if s:
+        if rel_to is not None:
+            rel_to = datetime.datetime.fromtimestamp(rel_to).timetuple()
+        else:
+            rel_to = None
+
+        t_struct, parsed_as = us_cal.parse(s, rel_to)
+
+        if not parsed_as:
+            t_struct, parsed_as = us_cal.parse(s, None)
+
+        if not parsed_as:
+            t_struct, parsed_as = native_cal.parse(s, rel_to)
+
+        if not parsed_as:
+            t_struct, parsed_as = native_cal.parse(s, None)
+
+        if parsed_as:
+            t_struct = list(t_struct)
+            if parsed_as == 1:
+                # only parsed as date, eliminate time part
+                t_struct[3:6] = (0, 0, 0)
+
+            t_struct[8] = -1  # isdst -- we don't know if that is DST
+
+            if use_utc:
+                from calendar import timegm
+                return timegm(t_struct)
+            else:
+                return time.mktime(t_struct)
+        else:
+            raise ValueError("failed to parse date/time '%s'" % s)
+    else:
+        return None
+
+
+def set_with_parsed_datetime(data, name, rel_to_name=None, use_utc=False):
+    data[name] = parse_datetime(
+            data[name],
+            data.get(rel_to_name),
+            use_utc=use_utc)
+
+# }}}
 
 
 # {{{ HTTP middleware
@@ -114,21 +167,25 @@ class DBSessionInjector(object):
                 mig_api.version_control(dburl, versioning_repo)
                 db_ver = mig_api.db_version(dburl, versioning_repo)
 
-
-            print "found database version %d, code uses version %d" % (db_ver, latest_ver)
+            print "found database version %d, code uses version %d" % (
+                    db_ver, latest_ver)
 
             if db_ver < latest_ver:
                 print "upgrading..."
                 mig_api.upgrade(dburl, versioning_repo)
         else:
-            # we're only just creating the db, thus using the latest available version
+            # we're only just creating the db, thus using the latest available
+            # version
             mig_api.version_control(dburl, versioning_repo, version=latest_ver)
 
         # }}}
 
         from sqlalchemy.pool import NullPool
         from sqlalchemy import create_engine
-        self.engine = create_engine(dburl, poolclass=NullPool)
+        self.engine = create_engine(
+                dburl, poolclass=NullPool,
+                echo=echo
+                )
 
         from synoptic.datamodel import DataModel
         self.datamodel = DataModel()
@@ -139,6 +196,10 @@ class DBSessionInjector(object):
         self.sessionmaker = sessionmaker(bind=self.engine, autoflush=True,
                 autocommit=False)
 
+        session = self.sessionmaker()
+        session.execute("analyze;")
+        session.commit()
+
         self.sub_app = sub_app
 
     def __call__(self, environ, start_response):
@@ -153,8 +214,6 @@ class DBSessionInjector(object):
             raise
 
 
-
-
 class ErrorMiddleware(object):
     def __init__(self, sub_app):
         self.sub_app = sub_app
@@ -170,7 +229,7 @@ class ErrorMiddleware(object):
             return e(environ, start_response)
         except:
             status = "500 Server Error"
-            response_headers = [("content-type","text/plain")]
+            response_headers = [("content-type", "text/plain")]
             exc_info = sys.exc_info()
             ex_type, ex_val, ex_tb = exc_info
 
@@ -181,13 +240,11 @@ class ErrorMiddleware(object):
                 pass
 
             start_response(status, response_headers, exc_info)
-            return ["%s: %s" % (ex_type.__name__,str(ex_val))]
+            return ["%s: %s" % (ex_type.__name__, str(ex_val))]
 
 from paste.wsgiwrappers import WSGIRequest, WSGIResponse
 
 
-
-
 class Response(WSGIResponse):
     def __init__(self, environ, start_response, *args, **kwargs):
         self.environ = environ
@@ -199,9 +256,6 @@ class Response(WSGIResponse):
         return WSGIResponse.__call__(self, self.environ, self.start_response)
 
 
-
-
-
 class Request(WSGIRequest):
     def __init__(self, environ, start_response):
         WSGIRequest.__init__(self, environ)
@@ -215,6 +269,7 @@ class Request(WSGIRequest):
 
 # }}}
 
+
 # {{{ URL dispatch
 
 class ApplicationBase(object):
@@ -264,8 +319,6 @@ class ApplicationBase(object):
 # }}}
 
 
-
-
 class Application(ApplicationBase):
     def __init__(self, urlprefix="/", allowed_networks=[]):
         import re
@@ -333,28 +386,32 @@ class Application(ApplicationBase):
 
     def get_json_items(self, session, model, parsed_query, max_timestamp):
         itemversions_query = query_itemversions(
-                session, model, parsed_query, max_timestamp).alias("currentitemversions")
+                session, model, parsed_query, max_timestamp)\
+                        .alias("currentitemversions")
 
         # prepare eager loading of tags
         from sqlalchemy.sql import select
         iv_and_t_query = select([itemversions_query, model.tags],
-                from_obj=[itemversions_query
+                from_obj=[
+                    itemversions_query
                     .outerjoin(model.itemversions_tags,
                         itemversions_query.c.id
-                        ==model.itemversions_tags.c.itemversion_id)
+                        == model.itemversions_tags.c.itemversion_id)
                     .outerjoin(model.tags,
                         model.itemversions_tags.c.tag_id
-                        ==model.tags.c.id)
+                        == model.tags.c.id)
                     ],
                 use_labels=True)
 
         last_id = None
         result = []
+
         for row in session.execute(iv_and_t_query):
             if last_id != row[itemversions_query.c.id]:
                 last_id = row[itemversions_query.c.id]
                 result.append(
-                        {"id": row[itemversions_query.c.item_id],
+                        {
+                            "id": row[itemversions_query.c.item_id],
                             "version_id": row[itemversions_query.c.id],
                             "contents": row[itemversions_query.c.contents],
                             "contents_html": ItemVersion.htmlize(
@@ -393,7 +450,8 @@ class Application(ApplicationBase):
                     .where(ItemVersion.tags.any(id=google_calendar_tag.id))
 
             import re
-            config_entry_re = re.compile(r"^\s*(?:\*\s+)?([a-zA-Z0-9]+)\s*:\s*(.*)\s*$",
+            config_entry_re = re.compile(
+                    r"^\s*(?:\*\s+)?([a-zA-Z0-9]+)\s*:\s*(.*)\s*$",
                     re.MULTILINE)
             for item_ver in dbsession.query(ItemVersion).from_statement(qry):
                 entry = {}
@@ -422,26 +480,28 @@ class Application(ApplicationBase):
 
         now = time()
 
-        from simplejson import dumps
         from sqlalchemy import asc, desc
         base_query = request.dbsession.query(ItemVersion)
         if list(base_query.limit(1)):
             return request.respond(
-                    dumps({
-                        "min": base_query.order_by(asc(ItemVersion.timestamp))[0].timestamp,
-                        "max": max(now,
-                               base_query.order_by(desc(ItemVersion.timestamp))[0].timestamp),
+                    json.dumps({
+                        "min":
+                        base_query.order_by(asc(ItemVersion.timestamp))[0].timestamp,
+                        "max":
+                        max(now,
+                            base_query.order_by(
+                                desc(ItemVersion.timestamp))[0].timestamp),
                         "now": now,
                         }),
                     mimetype="text/plain")
         else:
             return request.respond(
-                    dumps({ "min": now, "max": now+1, "now": now, }),
+                    json.dumps({"min": now, "max": now+1, "now": now, }),
                     mimetype="text/plain")
 
     def get_tags_with_usecounts(self, session, model, parsed_query=None,
             max_timestamp=None, startswith=None, limit=None):
-        from sqlalchemy.sql import select, and_, or_, not_, func
+        from sqlalchemy.sql import select, func
 
         if parsed_query is not None:
             itemversions_q = query_itemversions(session, model,
@@ -460,9 +520,11 @@ class Application(ApplicationBase):
                     model.tags.c.name,
                     func.count(itemversions_q.c.id).label("use_count"),
                         ],
-                    from_obj=[model.itemversions_tags
+                    from_obj=[
+                        model.itemversions_tags
                         .join(itemversions_q,
-                            itemversions_q.c.id==model.itemversions_tags.c.itemversion_id)
+                            itemversions_q.c.id
+                            == model.itemversions_tags.c.itemversion_id)
                         .join(model.tags)
                         ])
                 )
@@ -470,9 +532,10 @@ class Application(ApplicationBase):
         if startswith:
             twuc_q = twuc_q.where(model.tags.c.name.startswith(startswith))
 
-        twuc_q = (twuc_q
+        twuc_q = (
+                twuc_q
                 .group_by(model.tags.c.id)
-                .having(func.count(itemversions_q.c.id)>0)
+                .having(func.count(itemversions_q.c.id) > 0)
                 .order_by(model.tags.c.name)
                 )
 
@@ -489,9 +552,8 @@ class Application(ApplicationBase):
                 request.dbsession, request.datamodel,
                 startswith=query, limit=limit)
 
-        from simplejson import dumps
         return request.respond(
-                dumps([row.name for row in q]),
+                json.dumps([row.name for row in q]),
                 mimetype="text/plain")
 
     def http_get_tags_for_query(self, request):
@@ -516,20 +578,17 @@ class Application(ApplicationBase):
                 request.dbsession, request.datamodel, parsed_query,
                 max_timestamp)
 
-        from simplejson import dumps
-
         tags = [list(row) for row in result]
 
         return request.respond(
-                dumps({
-                "tags": tags,
-                "query_tags": query_tags,
-                }),
+                json.dumps({
+                    "tags": tags,
+                    "query_tags": query_tags,
+                    }),
                 mimetype="text/plain")
 
     def http_rename_tag(self, request):
-        from simplejson import loads, dumps
-        data = loads(request.POST["json"])
+        data = json.loads(request.POST["json"])
 
         old_name = data["old_name"]
         new_name = data["new_name"]
@@ -542,7 +601,7 @@ class Application(ApplicationBase):
         new_tag_query = request.dbsession.query(Tag).filter_by(name=new_name)
 
         if new_tag_query.count():
-            raise ValueError, "tag already exsits"
+            raise ValueError("tag already exsits")
 
         tag.name = new_name
 
@@ -561,7 +620,7 @@ class Application(ApplicationBase):
                 request.dbsession.query(ItemVersion)
                 .filter_by(item_id=int(request.GET["id"]))
                 .order_by(ItemVersion.timestamp.desc())
-                .filter(ItemVersion.contents != None)
+                .filter(ItemVersion.contents != None)  # noqa
                 .limit(1)
                     )
 
@@ -569,8 +628,7 @@ class Application(ApplicationBase):
             from paste.httpexceptions import HTTPNotFound
             raise HTTPNotFound()
 
-        from simplejson import dumps
-        return request.respond(dumps(self.item_to_json(query.first())),
+        return request.respond(json.dumps(self.item_to_json(query.first())),
                 mimetype="text/plain")
 
     def http_get_item_version_by_id(self, request):
@@ -583,8 +641,7 @@ class Application(ApplicationBase):
             from paste.httpexceptions import HTTPNotFound
             raise HTTPNotFound()
 
-        from simplejson import dumps
-        return request.respond(dumps(self.item_to_json(query.first())),
+        return request.respond(json.dumps(self.item_to_json(query.first())),
                 mimetype="text/plain")
 
     def http_get_items(self, request):
@@ -629,13 +686,11 @@ class Application(ApplicationBase):
 
             query_tags = []
 
-        from simplejson import dumps
-
         # kind of silly to do two JSON encodes, but what the heck
-        result_hash = hash(dumps(json_items))
+        result_hash = hash(json.dumps(json_items))
 
         # and ship them out by JSON
-        return request.respond(dumps({
+        return request.respond(json.dumps({
             "items": json_items,
             "result_hash": result_hash,
             "tags": tags,
@@ -658,11 +713,10 @@ class Application(ApplicationBase):
                 request.dbsession, request.datamodel,
                 parsed_query, max_timestamp)
 
-        from simplejson import dumps
-        result_hash = hash(dumps(json_items))
+        result_hash = hash(json.dumps(json_items))
 
         # and ship them out by JSON
-        return request.respond(dumps(result_hash),
+        return request.respond(json.dumps(result_hash),
                 mimetype="text/plain")
 
     def http_print_items(self, request):
@@ -694,56 +748,21 @@ class Application(ApplicationBase):
 
         json = [
                 {
-                    "version_id": iv.id, 
-                    "timestamp":iv.timestamp,
-                    "is_current":False,
-                    "contents":iv.contents,
+                    "version_id": iv.id,
+                    "timestamp": iv.timestamp,
+                    "is_current": False,
+                    "contents": iv.contents,
                     }
                 for iv in request.dbsession.query(ItemVersion)
                 .filter_by(item_id=item_id)
                 .order_by(ItemVersion.timestamp.desc())]
         json[0]["is_current"] = True
 
-        from simplejson import dumps
-        return request.respond(dumps(json),
+        return request.respond(json.dumps(json),
                 mimetype="text/plain")
 
-    @staticmethod
-    def parse_datetime(data, name, rel_to_name=None, use_utc=False):
-        import parsedatetime.parsedatetime as pdt
-        cal = pdt.Calendar()
-        import datetime
-        import time
-
-        if data[name]:
-            if rel_to_name is not None and data[rel_to_name] is not None:
-                rel_to = datetime.datetime.fromtimestamp(
-                        data[rel_to_name]).timetuple()
-            else:
-                rel_to = None
-
-            t_struct, parsed_as = cal.parse(data[name], rel_to)
-            if parsed_as:
-                t_struct = list(t_struct)
-                if parsed_as == 1:
-                    # only parsed as date, eliminate time part
-                    t_struct[3:6] = (0,0,0)
-
-                t_struct[8] = -1 # isdst -- we don't know if that is DST
-
-                if use_utc:
-                    from calendar import timegm
-                    data[name] = timegm(t_struct)
-                else:
-                    data[name] = time.mktime(t_struct)
-            else:
-                data[name] = None
-        else:
-            data[name] = None
-
     def http_store_item(self, request):
-        from simplejson import loads, dumps
-        data = loads(request.POST["json"])
+        data = json.loads(request.POST["json"])
 
         current_query = data.pop("current_query", None)
         deleting = data["contents"] is None
@@ -768,12 +787,12 @@ class Application(ApplicationBase):
             voh = None
 
         if not deleting:
-            self.parse_datetime(data, "start_date", 
+            set_with_parsed_datetime(data, "start_date",
                     use_utc=data["all_day"])
-            self.parse_datetime(data, "end_date", "start_date", 
+            set_with_parsed_datetime(data, "end_date", "start_date",
                     use_utc=data["all_day"])
-            self.parse_datetime(data, "hide_until", "start_date")
-            self.parse_datetime(data, "highlight_at", "start_date")
+            set_with_parsed_datetime(data, "hide_until", "start_date")
+            set_with_parsed_datetime(data, "highlight_at", "start_date")
 
         if not deleting:
             from synoptic.datamodel import is_valid_tag
@@ -784,29 +803,27 @@ class Application(ApplicationBase):
         itemversion = store_itemversion(request.dbsession,
                 **data)
 
-        request.dbsession.commit() # fills in the item_id
+        request.dbsession.commit()  # fills in the item_id
 
         if voh is not None:
             voh.insert(len(voh), itemversion.item_id)
             voh.save()
 
         # send response
-        from simplejson import dumps
         return request.respond(
-                dumps(self.item_to_json(itemversion)),
+                json.dumps(self.item_to_json(itemversion)),
                 mimetype="text/plain")
 
     def http_item_datebump(self, request):
-        from simplejson import loads, dumps
-        data = loads(request.POST["json"])
+        data = json.loads(request.POST["json"])
 
         bump_interval = data["bump_interval"]
         bump_direction = data["bump_direction"]
 
-        self.parse_datetime(data, "start_date")
-        self.parse_datetime(data, "end_date", "start_date")
-        self.parse_datetime(data, "hide_until", "start_date")
-        self.parse_datetime(data, "highlight_at", "start_date")
+        set_with_parsed_datetime(data, "start_date")
+        set_with_parsed_datetime(data, "end_date", "start_date")
+        set_with_parsed_datetime(data, "hide_until", "start_date")
+        set_with_parsed_datetime(data, "highlight_at", "start_date")
 
         tdelta = None
         increment_func = None
@@ -882,14 +899,12 @@ class Application(ApplicationBase):
             if data[key] is not None:
                 data[key] = increment_func(data[key])
 
-        from simplejson import dumps
         return request.respond(
-                dumps(data),
+                json.dumps(data),
                 mimetype="text/plain")
 
     def http_reorder_item(self, request):
-        from simplejson import loads
-        data = loads(request.POST["json"])
+        data = json.loads(request.POST["json"])
 
         from synoptic.query import parse_query
         from synoptic.datamodel import ViewOrderingHandler
@@ -926,15 +941,14 @@ class Application(ApplicationBase):
                 className=get_google_calendar_tag(entry_dict),
                 requireOnline=True))
 
-        from simplejson import dumps
         return request.respond(
-                dumps(event_sources),
+                json.dumps(event_sources),
                 mimetype="text/plain")
 
     def http_calendar_data(self, request):
         start = float(request.GET.get("start", 0))
         end = float(request.GET.get("end", 0))
-        max_timestamp = None # FIXME
+        max_timestamp = None  # FIXME
 
         from_obj = get_current_itemversions_join(
                 request.datamodel, max_timestamp)
@@ -1002,43 +1016,42 @@ class Application(ApplicationBase):
             start=now,
             end=now + 7.5 * 60,
             allDay=False,
-            className=["calendar-now" ],
+            className=["calendar-now"],
             ))
 
-        from simplejson import dumps
         return request.respond(
-                dumps(data),
+                json.dumps(data),
                 mimetype="text/plain")
 
     # }}}
 
     def http_get_all_js(self, request):
         all_js_filenames = [
-          "jquery.js",
-          "jquery-migrate.js",
-          "jquery.timers.js",
-          "jquery.bgiframe.js",
-          "jquery.dimensions.js",
-          "jquery.contextmenu.r2.js",
-          "jquery-ui.js",
+            "jquery.js",
+            "jquery-migrate.js",
+            "jquery.timers.js",
+            "jquery.bgiframe.js",
+            "jquery.dimensions.js",
+            "jquery.contextmenu.r2.js",
+            "jquery-ui.js",
 
-          # override because of z-index bug
-          # http://bugs.jqueryui.com/ticket/5479
-          "jquery.ui.datepicker.js", 
+            # override because of z-index bug
+            # http://bugs.jqueryui.com/ticket/5479
+            "jquery.ui.datepicker.js",
 
-          "inheritance.js",
-          "json2.js",
-          "rsh.js",
-          "sprintf.js",
-          ]
+            "inheritance.js",
+            "json2.js",
+            "rsh.js",
+            "sprintf.js",
+            ]
         sep = "/* %s */\n" % (75*"-")
         all_js = "".join(
             "%s/* %s */\n%s%s" % (sep, fn, sep, get_static_file(fn)[0])
             for fn in all_js_filenames)
 
         from synoptic.datamodel import BUMP_INTERVALS
-        from simplejson import dumps
-        all_js = "var BUMP_INTERVALS = %s;\n%s" % (dumps(BUMP_INTERVALS), all_js)
+        all_js = "var BUMP_INTERVALS = %s;\n%s" % (
+                json.dumps(BUMP_INTERVALS), all_js)
 
         return request.respond(all_js,
             mimetype="text/javascript")
@@ -1046,10 +1059,11 @@ class Application(ApplicationBase):
     def http_get_tag_color_css(self, request):
         is_calendar = request.GET.get("calendar", "") == "true"
         if is_calendar:
-            pattern = """.tag-%(tag)s, .fc-agenda .tag-%(tag)s .fc-event-time, .tag-%(tag)s .fc-event-inner
+            pattern = """.tag-%(tag)s, .fc-agenda .tag-%(tag)s \
+                    .fc-event-time, .tag-%(tag)s .fc-event-inner
                 { background-color: %(color)s; border-color: %(color)s; }"""
         else:
-            pattern = ".tag-%(tag)s { background-color: %(color)s; color:white; }" 
+            pattern = ".tag-%(tag)s { background-color: %(color)s; color:white; }"
 
         color_decls = []
         for entry_dict in self.get_config_info_by_tag(
@@ -1062,8 +1076,12 @@ class Application(ApplicationBase):
                     request.datamodel, request.dbsession, u"googlecalendar"):
                 if "color" in entry_dict:
                     color_decls.append(
-                        """.%(tag)s, .fc-agenda .%(tag)s .fc-event-time, .%(tag)s .fc-event-inner
-                            { background-color: %(color)s; border-color: %(color)s; }"""
+                        """.%(tag)s, .fc-agenda .%(tag)s .fc-event-time, \
+                                .%(tag)s .fc-event-inner
+                            {
+                                background-color: %(color)s;
+                                border-color: %(color)s;
+                            }"""
                         % dict(
                             tag=get_google_calendar_tag(entry_dict),
                             color=entry_dict["color"]))
